import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

from ..models.version import Version

T = TypeVar("T", bound="DocumentBookmarkCurrentVersion")


@attr.s(auto_attribs=True)
class DocumentBookmarkCurrentVersion:
    """ Document bookmark schema containing the current version info. """

    name: str
    item_id: int
    collection_id: int
    id: int
    created_on: datetime.datetime
    current_version: Version
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        item_id = self.item_id
        collection_id = self.collection_id
        id = self.id
        created_on = self.created_on.isoformat()

        current_version = self.current_version.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "item_id": item_id,
                "collection_id": collection_id,
                "id": id,
                "created_on": created_on,
                "current_version": current_version,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        item_id = d.pop("item_id")

        collection_id = d.pop("collection_id")

        id = d.pop("id")

        created_on = isoparse(d.pop("created_on"))

        current_version = Version.from_dict(d.pop("current_version"))

        document_bookmark_current_version = cls(
            name=name,
            item_id=item_id,
            collection_id=collection_id,
            id=id,
            created_on=created_on,
            current_version=current_version,
        )

        document_bookmark_current_version.additional_properties = d
        return document_bookmark_current_version

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.extract_paragraph_config import ExtractParagraphConfig
from ..models.extract_question_config import ExtractQuestionConfig
from ..models.extract_value_config import ExtractValueConfig
from ..types import UNSET, Unset

T = TypeVar("T", bound="SynchronousExtractCreate")


@attr.s(auto_attribs=True)
class SynchronousExtractCreate:
    """ Schema for simple extract task generation """

    file_id: int
    config: Union[Unset, ExtractValueConfig, ExtractParagraphConfig, ExtractQuestionConfig] = UNSET
    model_id: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        file_id = self.file_id
        config: Union[Unset, ExtractValueConfig, ExtractParagraphConfig, ExtractQuestionConfig]
        if isinstance(self.config, Unset):
            config = UNSET
        elif isinstance(self.config, ExtractValueConfig):
            config = UNSET
            if not isinstance(self.config, Unset):
                config = self.config.to_dict()

        elif isinstance(self.config, ExtractParagraphConfig):
            config = UNSET
            if not isinstance(self.config, Unset):
                config = self.config.to_dict()

        else:
            config = UNSET
            if not isinstance(self.config, Unset):
                config = self.config.to_dict()

        model_id = self.model_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "file_id": file_id,
            }
        )
        if config is not UNSET:
            field_dict["config"] = config
        if model_id is not UNSET:
            field_dict["model_id"] = model_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file_id = d.pop("file_id")

        def _parse_config(data: Any) -> Union[Unset, ExtractValueConfig, ExtractParagraphConfig, ExtractQuestionConfig]:
            data = None if isinstance(data, Unset) else data
            config: Union[Unset, ExtractValueConfig, ExtractParagraphConfig, ExtractQuestionConfig]
            try:
                config = UNSET
                _config = data
                if not isinstance(_config, Unset):
                    config = ExtractValueConfig.from_dict(_config)

                return config
            except:  # noqa: E722
                pass
            try:
                config = UNSET
                _config = data
                if not isinstance(_config, Unset):
                    config = ExtractParagraphConfig.from_dict(_config)

                return config
            except:  # noqa: E722
                pass
            config = UNSET
            _config = data
            if not isinstance(_config, Unset):
                config = ExtractQuestionConfig.from_dict(_config)

            return config

        config = _parse_config(d.pop("config", UNSET))

        model_id = d.pop("model_id", UNSET)

        synchronous_extract_create = cls(
            file_id=file_id,
            config=config,
            model_id=model_id,
        )

        synchronous_extract_create.additional_properties = d
        return synchronous_extract_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="DocType")


@attr.s(auto_attribs=True)
class DocType:
    """ Output doc type schema for indexing task details. """

    n_docs: int
    type: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        n_docs = self.n_docs
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "n_docs": n_docs,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        n_docs = d.pop("n_docs")

        type = d.pop("type")

        doc_type = cls(
            n_docs=n_docs,
            type=type,
        )

        doc_type.additional_properties = d
        return doc_type

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.extract_top_chunk_out import ExtractTopChunkOut
from ...models.http_validation_error import HTTPValidationError
from ...models.synchronous_extract_create import SynchronousExtractCreate
from ...types import Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    json_body: SynchronousExtractCreate,
) -> Dict[str, Any]:
    url = "{}/api/v1/extract/tasks/synchronous/".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[List[ExtractTopChunkOut], None, None, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = ExtractTopChunkOut.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[List[ExtractTopChunkOut], None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: SynchronousExtractCreate,
) -> Response[Union[List[ExtractTopChunkOut], None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: SynchronousExtractCreate,
) -> Optional[Union[List[ExtractTopChunkOut], None, None, HTTPValidationError]]:
    """Create and run a synchronous extract task.

    Only orgadmins are allowed to run synchronous extract tasks.
    The id of the organization is inferred from the session token.

    If model id is passed in the parameters, the task is created from the models.
    Otherwise, the seeds or the config to base the extract on must be passed in the body."""

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: SynchronousExtractCreate,
) -> Response[Union[List[ExtractTopChunkOut], None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: SynchronousExtractCreate,
) -> Optional[Union[List[ExtractTopChunkOut], None, None, HTTPValidationError]]:
    """Create and run a synchronous extract task.

    Only orgadmins are allowed to run synchronous extract tasks.
    The id of the organization is inferred from the session token.

    If model id is passed in the parameters, the task is created from the models.
    Otherwise, the seeds or the config to base the extract on must be passed in the body."""

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="Permission")


@attr.s(auto_attribs=True)
class Permission:
    """ Permission base schema. """

    action: str
    role: str
    product: str
    license_type: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        action = self.action
        role = self.role
        product = self.product
        license_type = self.license_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "action": action,
                "role": role,
                "product": product,
                "license_type": license_type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        action = d.pop("action")

        role = d.pop("role")

        product = d.pop("product")

        license_type = d.pop("license_type")

        permission = cls(
            action=action,
            role=role,
            product=product,
            license_type=license_type,
        )

        permission.additional_properties = d
        return permission

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.document_layout_item import DocumentLayoutItem

T = TypeVar("T", bound="DocumentLayoutOut")


@attr.s(auto_attribs=True)
class DocumentLayoutOut:
    """ "Document Layout schema for the GET route """

    layout: List[DocumentLayoutItem]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        layout = []
        for layout_item_data in self.layout:
            layout_item = layout_item_data.to_dict()

            layout.append(layout_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "layout": layout,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        layout = []
        _layout = d.pop("layout")
        for layout_item_data in _layout:
            layout_item = DocumentLayoutItem.from_dict(layout_item_data)

            layout.append(layout_item)

        document_layout_out = cls(
            layout=layout,
        )

        document_layout_out.additional_properties = d
        return document_layout_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.http_validation_error import HTTPValidationError
from ...models.metadata_create import MetadataCreate
from ...models.metadata_out import MetadataOut
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    json_body: MetadataCreate,
    folder_id: Union[Unset, int] = UNSET,
    version_id: Union[Unset, int] = UNSET,
    org_id: Union[Unset, int] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/metadata/".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "folder_id": folder_id,
        "version_id": version_id,
        "org_id": org_id,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[MetadataOut, None, None, None, None, HTTPValidationError]]:
    if response.status_code == 201:
        response_201 = MetadataOut.from_dict(response.json())

        return response_201
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 409:
        response_409 = None

        return response_409
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 404:
        response_404 = None

        return response_404
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[MetadataOut, None, None, None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: MetadataCreate,
    folder_id: Union[Unset, int] = UNSET,
    version_id: Union[Unset, int] = UNSET,
    org_id: Union[Unset, int] = UNSET,
) -> Response[Union[MetadataOut, None, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        folder_id=folder_id,
        version_id=version_id,
        org_id=org_id,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: MetadataCreate,
    folder_id: Union[Unset, int] = UNSET,
    version_id: Union[Unset, int] = UNSET,
    org_id: Union[Unset, int] = UNSET,
) -> Optional[Union[MetadataOut, None, None, None, None, HTTPValidationError]]:
    """Create a metadata and set its value to documents or folders.

    Only orgadmins and services can create metadata.

    If a service makes the query, the org_id parameter is expected,
    and the metadata is only created, no value is assigned to folders or versions.
    For this reason, in this case a value is not expected, but value_type is required.

    For orgadmins, the organization id is inferred from the session token, and:
    - If the version parameter is passed, the specified value is assigned to the version.
    - If the folder parameter is passed, the specified value is assigned to all of the
    documents in the folder and its children.
    - If neither is passed, the value is assigned to all documents in the organization.

    In any case, the metadata is created if it doesn't already exist.
    The metadata source is automatically set to user."""

    return sync_detailed(
        client=client,
        json_body=json_body,
        folder_id=folder_id,
        version_id=version_id,
        org_id=org_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: MetadataCreate,
    folder_id: Union[Unset, int] = UNSET,
    version_id: Union[Unset, int] = UNSET,
    org_id: Union[Unset, int] = UNSET,
) -> Response[Union[MetadataOut, None, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        folder_id=folder_id,
        version_id=version_id,
        org_id=org_id,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: MetadataCreate,
    folder_id: Union[Unset, int] = UNSET,
    version_id: Union[Unset, int] = UNSET,
    org_id: Union[Unset, int] = UNSET,
) -> Optional[Union[MetadataOut, None, None, None, None, HTTPValidationError]]:
    """Create a metadata and set its value to documents or folders.

    Only orgadmins and services can create metadata.

    If a service makes the query, the org_id parameter is expected,
    and the metadata is only created, no value is assigned to folders or versions.
    For this reason, in this case a value is not expected, but value_type is required.

    For orgadmins, the organization id is inferred from the session token, and:
    - If the version parameter is passed, the specified value is assigned to the version.
    - If the folder parameter is passed, the specified value is assigned to all of the
    documents in the folder and its children.
    - If neither is passed, the value is assigned to all documents in the organization.

    In any case, the metadata is created if it doesn't already exist.
    The metadata source is automatically set to user."""

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            folder_id=folder_id,
            version_id=version_id,
            org_id=org_id,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.document_layout_item_coordinates import DocumentLayoutItemCoordinates
from ..types import UNSET, Unset

T = TypeVar("T", bound="DocumentLayoutItem")


@attr.s(auto_attribs=True)
class DocumentLayoutItem:
    """ "Document Layout Item schema for the GET route """

    id: int
    elastic_id: str
    type: List[str]
    coordinates: DocumentLayoutItemCoordinates
    page: List[int]
    content: str
    related: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        elastic_id = self.elastic_id
        type = self.type

        coordinates = self.coordinates.to_dict()

        page = self.page

        content = self.content
        related = self.related

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "elastic_id": elastic_id,
                "type": type,
                "coordinates": coordinates,
                "page": page,
                "content": content,
            }
        )
        if related is not UNSET:
            field_dict["related"] = related

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        elastic_id = d.pop("elastic_id")

        type = cast(List[str], d.pop("type"))

        coordinates = DocumentLayoutItemCoordinates.from_dict(d.pop("coordinates"))

        page = cast(List[int], d.pop("page"))

        content = d.pop("content")

        related = d.pop("related", UNSET)

        document_layout_item = cls(
            id=id,
            elastic_id=elastic_id,
            type=type,
            coordinates=coordinates,
            page=page,
            content=content,
            related=related,
        )

        document_layout_item.additional_properties = d
        return document_layout_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="BBox")


@attr.s(auto_attribs=True)
class BBox:
    """  """

    x_min: float
    y_min: float
    x_max: float
    y_max: float
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        x_min = self.x_min
        y_min = self.y_min
        x_max = self.x_max
        y_max = self.y_max

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "x_min": x_min,
                "y_min": y_min,
                "x_max": x_max,
                "y_max": y_max,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        x_min = d.pop("x_min")

        y_min = d.pop("y_min")

        x_max = d.pop("x_max")

        y_max = d.pop("y_max")

        b_box = cls(
            x_min=x_min,
            y_min=y_min,
            x_max=x_max,
            y_max=y_max,
        )

        b_box.additional_properties = d
        return b_box

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.http_validation_error import HTTPValidationError
from ...models.metadata_out import MetadataOut
from ...models.metadata_source import MetadataSource
from ...models.metadata_type import MetadataType
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
    name: Union[Unset, str] = UNSET,
    source: Union[Unset, MetadataSource] = UNSET,
    value_type: Union[Unset, MetadataType] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/metadata/".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_source: Union[Unset, MetadataSource] = UNSET
    if not isinstance(source, Unset):
        json_source = source

    json_value_type: Union[Unset, MetadataType] = UNSET
    if not isinstance(value_type, Unset):
        json_value_type = value_type

    params: Dict[str, Any] = {
        "org_id": org_id,
        "limit": limit,
        "offset": offset,
        "name": name,
        "source": json_source,
        "value_type": json_value_type,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[List[MetadataOut], None, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = MetadataOut.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[List[MetadataOut], None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
    name: Union[Unset, str] = UNSET,
    source: Union[Unset, MetadataSource] = UNSET,
    value_type: Union[Unset, MetadataType] = UNSET,
) -> Response[Union[List[MetadataOut], None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        org_id=org_id,
        limit=limit,
        offset=offset,
        name=name,
        source=source,
        value_type=value_type,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
    name: Union[Unset, str] = UNSET,
    source: Union[Unset, MetadataSource] = UNSET,
    value_type: Union[Unset, MetadataType] = UNSET,
) -> Optional[Union[List[MetadataOut], None, HTTPValidationError]]:
    """List organization's metadata.

    Only basic users, orgadmins and services can have access to metadata.
    * If the user is a user, the id of the organization is guessed from the session token.
    * If the user is a service, the org_id query parameter is expected

    For users:
    * If filter is passed, the route only returns the metadata matching the filter.
    (e.g., filter=si will return size).
    * If type is specified, the route only returns the metadata of this type.
    * If source is specified, the route only returns the metadata from this source."""

    return sync_detailed(
        client=client,
        org_id=org_id,
        limit=limit,
        offset=offset,
        name=name,
        source=source,
        value_type=value_type,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
    name: Union[Unset, str] = UNSET,
    source: Union[Unset, MetadataSource] = UNSET,
    value_type: Union[Unset, MetadataType] = UNSET,
) -> Response[Union[List[MetadataOut], None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        org_id=org_id,
        limit=limit,
        offset=offset,
        name=name,
        source=source,
        value_type=value_type,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
    name: Union[Unset, str] = UNSET,
    source: Union[Unset, MetadataSource] = UNSET,
    value_type: Union[Unset, MetadataType] = UNSET,
) -> Optional[Union[List[MetadataOut], None, HTTPValidationError]]:
    """List organization's metadata.

    Only basic users, orgadmins and services can have access to metadata.
    * If the user is a user, the id of the organization is guessed from the session token.
    * If the user is a service, the org_id query parameter is expected

    For users:
    * If filter is passed, the route only returns the metadata matching the filter.
    (e.g., filter=si will return size).
    * If type is specified, the route only returns the metadata of this type.
    * If source is specified, the route only returns the metadata from this source."""

    return (
        await asyncio_detailed(
            client=client,
            org_id=org_id,
            limit=limit,
            offset=offset,
            name=name,
            source=source,
            value_type=value_type,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="ExtractConfigOut")


@attr.s(auto_attribs=True)
class ExtractConfigOut:
    """ Extract configuration schema to be returned from the API. """

    n_proposed_paragraphs: int
    n_auto_paragraphs: int
    org_id: int
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        n_proposed_paragraphs = self.n_proposed_paragraphs
        n_auto_paragraphs = self.n_auto_paragraphs
        org_id = self.org_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "n_proposed_paragraphs": n_proposed_paragraphs,
                "n_auto_paragraphs": n_auto_paragraphs,
                "org_id": org_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        n_proposed_paragraphs = d.pop("n_proposed_paragraphs")

        n_auto_paragraphs = d.pop("n_auto_paragraphs")

        org_id = d.pop("org_id")

        extract_config_out = cls(
            n_proposed_paragraphs=n_proposed_paragraphs,
            n_auto_paragraphs=n_auto_paragraphs,
            org_id=org_id,
        )

        extract_config_out.additional_properties = d
        return extract_config_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="ChunkBookmarkGetDivIdxs")


@attr.s(auto_attribs=True)
class ChunkBookmarkGetDivIdxs:
    """  """

    additional_properties: Dict[str, List[None]] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = []
            for additional_property_item_data in prop:
                additional_property_item = None

                field_dict[prop_name].append(additional_property_item)

        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        chunk_bookmark_get_div_idxs = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = []
            _additional_property = prop_dict
            for additional_property_item_data in _additional_property:
                additional_property_item = None

                additional_property.append(additional_property_item)

            additional_properties[prop_name] = additional_property

        chunk_bookmark_get_div_idxs.additional_properties = additional_properties
        return chunk_bookmark_get_div_idxs

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> List[None]:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: List[None]) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.get_version_styled_html_pages_response_get_version_styled_html_pages_api_v1_files_versions_version_id_styled_html_get import (
    GetVersionStyledHtmlPagesResponseGetVersionStyledHtmlPagesApiV1FilesVersionsVersionIdStyledHtmlGet,
)
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    version_id: int,
    page_nums: List[int],
) -> Dict[str, Any]:
    url = "{}/api/v1/files/versions/{version_id}/styled_html/".format(client.base_url, version_id=version_id)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_page_nums = page_nums

    params: Dict[str, Any] = {
        "page_nums": json_page_nums,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        GetVersionStyledHtmlPagesResponseGetVersionStyledHtmlPagesApiV1FilesVersionsVersionIdStyledHtmlGet,
        None,
        None,
        None,
        None,
        HTTPValidationError,
    ]
]:
    if response.status_code == 200:
        response_200 = GetVersionStyledHtmlPagesResponseGetVersionStyledHtmlPagesApiV1FilesVersionsVersionIdStyledHtmlGet.from_dict(
            response.json()
        )

        return response_200
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 404:
        response_404 = None

        return response_404
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 400:
        response_400 = None

        return response_400
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        GetVersionStyledHtmlPagesResponseGetVersionStyledHtmlPagesApiV1FilesVersionsVersionIdStyledHtmlGet,
        None,
        None,
        None,
        None,
        HTTPValidationError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    version_id: int,
    page_nums: List[int],
) -> Response[
    Union[
        GetVersionStyledHtmlPagesResponseGetVersionStyledHtmlPagesApiV1FilesVersionsVersionIdStyledHtmlGet,
        None,
        None,
        None,
        None,
        HTTPValidationError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        version_id=version_id,
        page_nums=page_nums,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    version_id: int,
    page_nums: List[int],
) -> Optional[
    Union[
        GetVersionStyledHtmlPagesResponseGetVersionStyledHtmlPagesApiV1FilesVersionsVersionIdStyledHtmlGet,
        None,
        None,
        None,
        None,
        HTTPValidationError,
    ]
]:
    """Get one page of the html representation of a file version.

    The HTML representation of the version is generated during indexing.

    Consequently, it may be unavailable until indexing is done.
    - Basic users can only access files in folders they have rights on.
    - Orgadmins can access files in all of their organization's folders."""

    return sync_detailed(
        client=client,
        version_id=version_id,
        page_nums=page_nums,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    version_id: int,
    page_nums: List[int],
) -> Response[
    Union[
        GetVersionStyledHtmlPagesResponseGetVersionStyledHtmlPagesApiV1FilesVersionsVersionIdStyledHtmlGet,
        None,
        None,
        None,
        None,
        HTTPValidationError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        version_id=version_id,
        page_nums=page_nums,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    version_id: int,
    page_nums: List[int],
) -> Optional[
    Union[
        GetVersionStyledHtmlPagesResponseGetVersionStyledHtmlPagesApiV1FilesVersionsVersionIdStyledHtmlGet,
        None,
        None,
        None,
        None,
        HTTPValidationError,
    ]
]:
    """Get one page of the html representation of a file version.

    The HTML representation of the version is generated during indexing.

    Consequently, it may be unavailable until indexing is done.
    - Basic users can only access files in folders they have rights on.
    - Orgadmins can access files in all of their organization's folders."""

    return (
        await asyncio_detailed(
            client=client,
            version_id=version_id,
            page_nums=page_nums,
        )
    ).parsed
